package	jp.co.sds_corp.hibiya.server;

import java.text.DecimalFormat;

import jp.co.sds_corp.util.common.CmnDate;
import jp.co.sds_corp.util.config.IConfiguration;
import jp.co.sds_corp.util.device.DeviceManager;
import jp.co.sds_corp.util.device.MemoryUsage;

/**
メモリの状況を監視します。
<BR>
XMLは以下の形式です。
<PRE>
&lt;Service name="Memory1" class="jp.co.sds_corp.hibiya.server.MemoryChecker"&gt;
	&lt;Minimum&gt;20&lt;/Minimum&gt;
	&lt;IncludeCache&gt;true&lt;/IncludeCache&gt;
</PRE>
Minimum要素では、最小の残り容量をパーセントで指定してください。
メモリの残り容量（パーセント）がこの値より小さくなるとエラーです。<BR>
キャッシュを空きメモリとしてカウントする場合には、IncludeCache要素をtrueに設定してください。
(Linuxの場合、空きメモリがキャッシュに割り当てられます。)
<BR>
Action要素については、{@link jp.co.sds_corp.hibiya.action.IAction}
の説明を参照してください。
*/
public class MemoryChecker extends AbstractChecker
{
	private static final long serialVersionUID = 1L;
	DecimalFormat   df = new DecimalFormat( "#,###" );
	DecimalFormat   df2 = new DecimalFormat( "#.0#" );
	String	drive;
	int		minimum;
	boolean	includeCache;	// キャッシュフラグ

	public MemoryChecker() {
		setType( "Memory" );
	}

	/**
	* 設定情報で初期化します。
	* @param	config	設定情報
	*/
	public void init( IConfiguration config ) throws Exception {
		super.init( config );
		minimum = config.getInteger( "Minimum" );
		includeCache = false;
		String var = config.get( "IncludeCache", "" );
		if ( var.equals( "true" )) {
			includeCache = true;
		}
	}

	/**
	* 監視処理を実行します。
	*/
	public void check() {
		try {
			CmnDate startTime = new CmnDate();
			MemoryUsage info = DeviceManager.getMemoryUsage();
			CmnDate endTime = new CmnDate();
			clearResult();
			setResponseTime( startTime, endTime );

			putResult( "Max", df.format( info.getMaxSize() ));
			putResult( "Used", df.format( info.getUsedSize() ));
			putResult( "Free", df.format( info.getFreeSize() ));
			putResult( "Cache", df.format( info.getCacheSize() ));
			putResult( "MaxSwap", df.format( info.getMaxSwapSize() ));
			putResult( "UsedSwap", df.format( info.getUsedSwapSize() ));
			putResult( "FreeSwap", df.format( info.getFreeSwapSize() ));

			long free = info.getFreeSize();
			if ( includeCache ) {
				free += info.getCacheSize();
			}
			double total = (double )info.getMaxSize();
			double rest = (double )free / total * 100.0;
			if ( total == 0.0 ) {
				putResult( "rest", "0%" );
			} else {
				putResult( "rest", df2.format( rest ) + "%" );
			}

			if ( rest < minimum ) {
				setException( new Exception( "no more memory"
								+ " rest=" + df2.format( rest )));
				setCheckerStatus( IChecker.Status.NG );
			} else {
				setCheckerStatus( IChecker.Status.OK );
			}
		} catch ( Throwable ex ) {
			synchronized( this ) {
				clearResult();
				setException( ex );
				setCheckerStatus( IChecker.Status.NG );
			}
		}
	}

	/**
	* 最小使用可能サイズ(minimum)を設定します。
	* @param    var     設定するデータ
	*/
	public void setMinimum( int var ) {
		this.minimum = var ;
	}
	/**
	* 最小使用可能サイズ(minimum)を取得します。
	* @return   minimumの内容
	*/
	public int getMinimum() {
		return minimum;
	}

	/**
	* キャッシュフラグ(includeCache)を設定します。
	* @param	var		設定するデータ
	*/
	public void setIncludeCache( boolean var ) {
		this.includeCache = var;
	}
	/**
	* キャッシュフラグ(includeCache)を取得します。
	* @return	includeCacheの内容
	*/
	public boolean getIncludeCache() {
		return includeCache;
	}

	/**
	* 設定されている内容をXML形式で出力します。
	* @param	config	設定情報
	*/
	public void saveConfig( IConfiguration config ) {
		super.saveConfig( config );
		config.addChild( "Minimum", String.valueOf( minimum ));
		if ( includeCache ) {
			config.addChild( "IncludeCache", "true" );
		}
	}

	/**
	* このモニタの文字列表現を返します。
	* @return	このモニタの文字列表現
	*/
	public String toString() {
		String ret = super.toString();
		ret += "["
			+ "min=" + minimum + ";"
			+ "cache=" + includeCache + ";"
			+ "]";
		return ret;
	}

}
